/*
 * A simple program to display and update a sprite using key strokes
 * Uses getKeyStates() method on a GameCanvas object
 */

package gamelibrary.game;

/**
 *
 * @author Griff
 */
import javax.microedition.lcdui.*;
import javax.microedition.lcdui.game.*;
import java.util.Vector;
import gameactual.*;
import gamelibrary.gameobject.*;
import gamelibrary.collisiondetection.*;

public class MyCanvas extends GameCanvas implements Runnable{
    private boolean running = true;
    private Coordinates viewCoords;
    private Coordinates maxScreenCoords;

    private Thread thread;
    private Background bg;
    private GameObject player;
    private Foe foe;
    private Maze maze;
    private LayerManager layerManager;

    private Vector gameObjects;
    private Camera camera;

    /*
     *  Constructor: must call super(boolean)
     */
    public MyCanvas(){
        super(true);    // true = suppress sending key events
    }

    public void start(){
        gameObjects = new Vector();
        viewCoords = new Coordinates(0, 0);
        maxScreenCoords = new Coordinates(200, 200);
        camera = new Camera(getWidth(), getHeight(), viewCoords, maxScreenCoords);
        layerManager = new LayerManager();
        bg = new Background(new BasicGameObject());
        maze = new Maze(maxScreenCoords);
        maze.addToList(gameObjects);
        Coordinates coordinates = new Coordinates(29, 169);
        player = new Player(new BasicGameObject(), coordinates);
        GameObject gameObject1 = new Lethal(new BasicGameObject());
        foe = new Foe(gameObject1, (new Coordinates(160, 30)), 1, player);
        try{
            thread = new Thread(this);
            thread.start();
        }
        catch(Exception e){
            System.out.println("Error loading sprite!!");
        }
        gameObjects.addElement(player);
        gameObjects.addElement(foe);
        gameObjects.addElement(bg);
        foe.addToLayerManager(layerManager);
        player.addToLayerManager(layerManager);
        maze.addToLayerManager(layerManager);
        bg.addToLayerManager(layerManager);
    }

   /*
    * Method to draw the updated display
    */
   public void gameDraw(Graphics g){
       bg.update();
       layerManager.paint(g, viewCoords.getX(), viewCoords.getY());
       flushGraphics();
   }

   public void stop(){
       running = false;
   }
   
   /*
    * The game loop polls for key states and updates the
    * sprite's position
    */
   public void run(){
       Graphics g = getGraphics();
       while(true){
           for (int i = 0; i < gameObjects.size(); i++){
               GameObject gameObjectI = (GameObject) gameObjects.elementAt(i);
               gameObjectI.acceptInput(getKeyStates());
               for (int j = 0; j < gameObjects.size(); j++){
                   GameObject gameObjectJ = (GameObject) gameObjects.elementAt(j);
                   if (i != j){
                        if (gameObjectI.collidesWith(gameObjectJ)){
                            gameObjectI.resolveCollision(gameObjectJ);
                            gameObjectJ.resolveCollision(gameObjectI);
                        }
                    }
               }
           }
           camera.trackGameObject(player);
           foe.chase();
           gameDraw(g);
           try{
                Thread.sleep(10);
           }
           catch(InterruptedException e){}
       }
    }
 }